Imports System.ComponentModel
Imports System.Drawing
Imports System.Windows.Forms
Imports System.Resources

Namespace DataGridWithImages

    '***********************************************************************************
    '**                                                                               **												   
    '**  Function   : public class DataGridImageColumn                                **
    '**  Last change: 26.10.2001                                                      **
    '**                                                                               **
    '**  Remarks    : for displaying bitmaps in a data grid, a new                    **
    '**               DataGridColumnStyle is needed.                                  **
    '**                                                                               **
    '***********************************************************************************
    ' <summary>
    ' Remarks: for displaying bitmaps in a data grid, a new DataGridColumnStyle is needed.
    ' </summary>

    Public Class DataGridImageColumn
        Inherits DataGridColumnStyle

        '***********************************************************************************
        '**                                                                               **												   
        '**  Function   : public class DataGridImageColumn                                **
        '**  Last change: 26.10.2001                                                      **
        '**                                                                               **
        '**  Remarks    : the constructor of the class                                    **
        '**                                                                               **
        '***********************************************************************************
        ' <summary>
        ' Remarks: the constructor of the class
        ' </summary>

        Public Sub New(ByVal pcol As PropertyDescriptor)

        End Sub

        '------------------------------------------------------------------------------------
        '------------------------------------------------------------------------------------
        '------------------------------------------------------------------------------------
        ' ALL THESE METHODS MUST BE OVERRIDDEN FROM 'DataGridColumnStyle'
        '------------------------------------------------------------------------------------
        '------------------------------------------------------------------------------------
        '------------------------------------------------------------------------------------

        '***********************************************************************************
        '**                                                                               **												   
        '**  Function   : protected overrides subAbort(int RowNum)                       **
        '**  Last change: 26.10.2001                                                      **
        '**                                                                               **
        '**  Remarks    : read-only, so nothing must be aborted                           **
        '**                                                                               **
        '***********************************************************************************
        ' <summary>
        ' Remarks: read-only, so nothing must be aborted
        ' </summary>

        Protected Overrides Sub Abort(ByVal RowNum As Integer)

        End Sub

        '***********************************************************************************
        '**                                                                               **												   
        '**  Function   : protected override bool Commit(CurrencyManager DataSource,      **
        '**                                              int RowNum)                      **
        '**  Last change: 26.10.2001                                                      **
        '**                                                                               **
        '**  Remarks    : read-only, so nothing must be committed                         **
        '**                                                                               **
        '***********************************************************************************
        ' <summary>
        ' Remarks: read-only, so nothing must be committed
        ' </summary>

        Protected Overrides Function Commit(ByVal DataSource As CurrencyManager, ByVal RowNum As Integer) As Boolean
            Return True
        End Function

        '***********************************************************************************
        '**                                                                               **												   
        '**  Function   : protected overrides subEdit(CurrencyManager Source ,int Rownum,**
        '**                                            Rectangle Bounds, bool ReadOnly,   **
        '**                                            string InstantText,                **
        '**                                            bool CellIsVisible)                **
        '**  Last change: 26.10.2001                                                      **
        '**                                                                               **
        '**  Remarks    : read-only, so nothing must could be edited                      **
        '**                                                                               **
        '***********************************************************************************
        ' <summary>
        ' Remarks: read-only, so nothing must could be edited
        ' </summary>

        Protected Overloads Overrides Sub Edit(ByVal Source As CurrencyManager, ByVal Rownum As Integer, ByVal Bounds As Rectangle, ByVal IsReadOnly As Boolean, ByVal InstantText As String, ByVal CellIsVisible As Boolean)

        End Sub

        '***********************************************************************************
        '**                                                                               **												   
        '**  Function   : protected override int GetMinimumHeight()                       **
        '**  Last change: 26.10.2001                                                      **
        '**                                                                               **
        '**  Remarks    : returns the minimum height of the picture                       **
        '**                                                                               **
        '***********************************************************************************
        ' <summary>
        ' Remarks: returns the minimum height of the picture
        ' </summary>

        Protected Overrides Function GetMinimumHeight() As Integer

            '
            ' return here your minimum height
            '
            Return 60
        End Function

        '***********************************************************************************
        '**                                                                               **												   
        '**  Function   : protected override int GetPreferredHeight(Graphics g,           **
        '**                                                         object   Value)       **
        '**  Last change: 26.10.2001                                                      **
        '**                                                                               **
        '**  Remarks    : returns the preferred height of the picture                     **
        '**                                                                               **
        '***********************************************************************************
        ' <summary>
        ' Remarks: returns the preferred height of the picture
        ' </summary>

        Protected Overrides Function GetPreferredHeight(ByVal g As Graphics, ByVal value As Object) As Integer

            '
            ' return here your preferred height
            '
            Return 60
        End Function

        '***********************************************************************************
        '**                                                                               **												   
        '**  Function   : protected override Size GetPreferredSize(Graphics g,            **
        '**                                                        object   Value)        **
        '**  Last change: 26.10.2001                                                      **
        '**                                                                               **
        '**  Remarks    : returns the preferred size of the picture                       **
        '**                                                                               **
        '***********************************************************************************
        ' <summary>
        ' Remarks: returns the preferred size of the picture
        ' </summary>

        Protected Overrides Function GetPreferredSize(ByVal g As Graphics, ByVal Value As Object) As Size

            '
            ' return here your preferred size
            '
            Dim PicSize As Size = New Size(100, 60)
            Return PicSize
        End Function

        '***********************************************************************************
        '**                                                                               **												   
        '**  Function   : protected overrides subPaint( ... )                            **
        '**  Last change: 26.10.2001                                                      **
        '**                                                                               **
        '**  Remarks    : the paint method of the 'DataGridColumnStyle' class do the      **
        '**               work. There exist three overloaded versions of me method      **
        '**                                                                               **
        '***********************************************************************************
        ' <summary>
        ' Remarks: the paint method of the 'DataGridColumnStyle' class do the work. There 
        '          exist three overloaded versions of me method
        ' </summary>

        Protected Overloads Overrides Sub Paint(ByVal g As Graphics, ByVal Bounds As Rectangle, ByVal Source As CurrencyManager, ByVal RowNum As Integer)

            Dim BackBrush As SolidBrush = New SolidBrush(Color.White)

            Dim ImagePic As Bitmap = ExtractImageFromOleContainer(CType(GetColumnValueAtRow(Source, RowNum), Byte()))
            g.FillRectangle(BackBrush, Bounds.X, Bounds.Y, Bounds.Width, Bounds.Height)
            g.DrawImage(CType(ImagePic, Image), Bounds.X + ((Bounds.Width - ImagePic.Width) \ 2), Bounds.Y, ImagePic.Width, ImagePic.Height)
        End Sub

        Protected Overloads Overrides Sub Paint(ByVal g As Graphics, ByVal Bounds As Rectangle, ByVal Source As CurrencyManager, ByVal RowNum As Integer, ByVal AlignToRight As Boolean)

            Dim BackBrush As SolidBrush = New SolidBrush(Color.White)

            Dim ImagePic As Bitmap = ExtractImageFromOleContainer(CType(GetColumnValueAtRow(Source, RowNum), Byte()))
            g.FillRectangle(BackBrush, Bounds.X, Bounds.Y, Bounds.Width, Bounds.Height)
            g.DrawImage(CType(ImagePic, Image), Bounds.X + ((Bounds.Width - ImagePic.Width) \ 2), Bounds.Y, ImagePic.Width, ImagePic.Height)
        End Sub

        Protected Overloads Sub Paint(ByVal g As Graphics, ByVal Bounds As Rectangle, ByVal Source As CurrencyManager, ByVal RowNum As Integer, ByVal BackBrush As Brush, ByVal ForeBrush As Brush, ByVal AlignToRight As Boolean)

            Dim ImagePic As Bitmap = ExtractImageFromOleContainer(CType(GetColumnValueAtRow(Source, RowNum), Byte()))
            g.FillRectangle(BackBrush, Bounds.X, Bounds.Y, Bounds.Width, Bounds.Height)
            g.DrawImage(CType(ImagePic, Image), Bounds.X + ((Bounds.Width - ImagePic.Width) \ 2), Bounds.Y, ImagePic.Width, ImagePic.Height)
        End Sub

        '***********************************************************************************
        '**                                                                               **												   
        '**  Function   : private Bitmap ExtractImageFromOleContainer(byte() Data)        **
        '**  Last change: 26.10.2001                                                      **
        '**                                                                               **
        '**  Remarks    : extract bitmap from OLE container                               **
        '**                                                                               **
        '***********************************************************************************
        ' <summary>
        ' Remarks: extract bitmap from OLE container
        ' </summary>
        Private Function ExtractImageFromOleContainer(ByVal Data() As Byte) As Bitmap

            Dim rm As ResourceManager = New ResourceManager("DGImageCol.Resource", Me.GetType().Assembly)
            Dim ImageArray(7999) As Byte
            Dim stream As System.IO.MemoryStream = Nothing
            Dim Picture As Bitmap

            ' &h1C15 = OLEContainer signature
            If (Data(0) = &H15 And Data(1) = &H1C) Then
                ' extract bitmap data from container
                Try
                    System.Buffer.BlockCopy(Data, &H48, ImageArray, 0, Data.Length - &H48)
                    stream = New System.IO.MemoryStream(ImageArray)
                    Picture = New Bitmap(stream)
                Catch
                    Picture = CType(rm.GetObject("Error"), Bitmap)
                End Try
                Return Picture
            Else
                ' if header incorrect, return default bitmap
                Return CType(rm.GetObject("Error"), Bitmap)
            End If
        End Function
    End Class
End Namespace
