Imports System.Windows.Forms
Imports System.Timers

Public Enum GridSelection As Integer
    Cell = 1
    Row = 2
End Enum

Public Class DataGridEx
    Inherits DataGrid
    Protected WithEvents objTimer As System.Windows.Forms.Timer

    Protected protCustomColumnsHeaders As Boolean
    Protected protOldHeaderFont As Font
    Protected protNRowsInCaption As Integer
    Protected protRowHeaderWidth As Integer

    Private components As System.ComponentModel.IContainer

    Private SelMode As GridSelection = GridSelection.Cell

    Public Property SelectionMode() As GridSelection
        Get
            Return SelMode
        End Get
        Set(ByVal Value As GridSelection)
            SelMode = Value
            If SelMode = GridSelection.Row Then
                Me.RowHeaderWidth = Me.protRowHeaderWidth
                Me.RowHeadersVisible = True
            End If
        End Set
    End Property

    Public Sub HitCellTest(ByRef Row As Integer, ByRef Col As Integer)
        Dim myGrid As DataGrid = Me
        Dim p As Point = Me.PointToClient(Me.MousePosition)
        Dim myHitInfo As DataGrid.HitTestInfo = myGrid.HitTest(p.X, p.Y)

        Row = myHitInfo.Row
        Col = myHitInfo.Column
        If myHitInfo.Row >= 0 And myHitInfo.Column >= 0 Then
            If SelMode = GridSelection.Row Then
                Me.Select(myHitInfo.Row)
            End If
        End If
    End Sub

    Public Sub HitCellTest()
        Dim myGrid As DataGrid = Me
        Dim p As Point = Me.PointToClient(Me.MousePosition)
        Dim myHitInfo As DataGrid.HitTestInfo = myGrid.HitTest(p.X, p.Y)

        If myHitInfo.Row >= 0 And myHitInfo.Column >= 0 Then
            If SelMode = GridSelection.Row Then
                Me.Select(myHitInfo.Row)
            End If
            RaiseEvent CellHitTest(Me, New CellSelectedEventArgs(myHitInfo.Row, myHitInfo.Column))
        End If
    End Sub

    Private Sub DataGridEx_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles MyBase.MouseUp
        Try
            If SelMode = GridSelection.Row And Me.CurrentCell.RowNumber >= 0 Then
                Me.Select(Me.CurrentCell.RowNumber)
            End If
            RaiseEvent CellSelected(Me, New CellSelectedEventArgs(Me.CurrentCell.RowNumber, Me.CurrentCell.ColumnNumber))
        Catch ex As Exception
        End Try
    End Sub

    Public Delegate Sub CellDelegate(ByVal sender As Object, ByVal e As CellSelectedEventArgs)

    Public Event CellSelected As CellDelegate
    Public Event MouseOverNotification As CellDelegate
    Public Event CellHitTest As CellDelegate

    Public Sub SetColumnSize(ByVal Table As DataTable, ByVal Column As Integer, ByVal ColumnSize As Integer)
        Dim tblStyle As DataGridTableStyle = GetTblStyle(Table)
        Try
            tblStyle.GridColumnStyles(Column).Width = ColumnSize
        Catch ex As Exception
        End Try
    End Sub

    Public Sub SetColumnSize(ByVal Table As DataTable, ByVal ColumnName As String, ByVal ColumnSize As Integer)
        Dim tblStyle As DataGridTableStyle = GetTblStyle(Table)
        Try
            tblStyle.GridColumnStyles(ColumnName).Width = ColumnSize
        Catch ex As Exception
        End Try
    End Sub

    Public Sub SetColumnName(ByVal Table As DataTable, ByVal Column As Integer, ByVal HeaderText As String)
        Dim tblStyle As DataGridTableStyle = GetTblStyle(Table)
        Try
            tblStyle.GridColumnStyles(Column).HeaderText = HeaderText
        Catch ex As Exception
        End Try
    End Sub

    Public Sub SetColumnName(ByVal Table As DataTable, ByVal ColumnName As String, ByVal HeaderText As String)
        Dim tblStyle As DataGridTableStyle = GetTblStyle(Table)
        Try
            tblStyle.GridColumnStyles(ColumnName).HeaderText = HeaderText
        Catch ex As Exception
        End Try
    End Sub

    Public Function GetTblStyle(ByVal Table As DataTable) As DataGridTableStyle
        Dim tblStyle As DataGridTableStyle
        tblStyle = Me.TableStyles(Table.TableName)
        If tblStyle Is Nothing Then
            tblStyle = CreateTableStyle(Table)
            Me.TableStyles.Add(tblStyle)
        End If
        Return tblStyle
    End Function

    Public Sub AdjustColumnWidths(ByVal Table As DataTable)
        AdjustColumnWidthToTitles(Table)
        Dim i As Integer
        For i = 0 To Table.Columns.Count - 1
            AdjustColumnToOptimumSize(Table, i)
        Next
    End Sub

    Public Sub AdjustColumnWidthToTitles(ByVal Table As DataTable)
        Dim tblStyle As DataGridTableStyle
        tblStyle = GetTblStyle(Table)
        Dim i As Integer
        Dim cs As DataGridColumnStyle
        Dim g As Graphics
        g = Me.CreateGraphics()
        Dim f As Font = Me.HeaderFont
        If Me.CustomColumnHeaders Then
            f = Me.protOldHeaderFont
        End If
        For Each cs In tblStyle.GridColumnStyles
            If cs.Width <> 0 Then
                If Me.CustomColumnHeaders Then
                    cs.Width = GetHeaderWidth(g, cs.HeaderText) + 10
                Else
                    cs.Width = CInt(g.MeasureString(cs.HeaderText, f).Width + 10)
                End If
            End If
        Next
        g.Dispose()
    End Sub

    Private Sub AdjustColumnToOptimumSize(ByVal Table As DataTable, ByVal ColumnNumber As Integer)
        Try
            Dim cn As Integer = ColumnNumber
            Dim tbl As DataTable
            If Not Table Is Nothing Then
                tbl = Table
            Else
                If TypeOf Me.DataSource Is DataTable Then
                    tbl = CType(Me.DataSource, DataTable)
                ElseIf TypeOf Me.DataSource Is DataView Then
                    tbl = CType(Me.DataSource, DataView).Table
                Else
                    Return
                End If
            End If
            If tbl.Columns(cn).DataType Is GetType(String) Then
                Dim i, l As Integer
                Dim tblStyle As DataGridTableStyle
                For Each tblStyle In Me.TableStyles
                    If tblStyle.MappingName = tbl.TableName Then
                        Exit For
                    End If
                Next
                If Not tblStyle Is Nothing Then
                    If tblStyle.MappingName <> tbl.TableName Then
                        tblStyle = Nothing
                    End If
                End If
                If tblStyle Is Nothing Then
                    tblStyle = CreateTableStyle(tbl)
                End If
                l = tblStyle.GridColumnStyles(tbl.Columns(cn).ColumnName).Width
                Dim g As Graphics
                g = Me.CreateGraphics()
                Dim f As Font = Me.Font
                For i = 0 To tbl.Rows.Count - 1
                    l = CInt(System.Math.Max(l, g.MeasureString(CStr(tbl.Rows(i).Item(cn)), f).Width))
                Next
                g.Dispose()
                l += 10
                If l > tblStyle.GridColumnStyles(tbl.Columns(cn).ColumnName).Width Then
                    tblStyle.GridColumnStyles(tbl.Columns(cn).ColumnName).Width = l
                End If
            End If
        Catch ex As Exception
        End Try
    End Sub

    Public Function CreateTableStyle(ByVal Table As DataTable) As DataGridTableStyle
        Dim ts1 As New DataGridTableStyle()
        ts1.MappingName = Table.TableName
        Dim i As Integer
        For i = 0 To Table.Columns.Count - 1
            If Table.Columns(i).DataType Is GetType(Boolean) Then
                Dim boolCol As New DataGridBoolColumnEx()
                boolCol.MappingName = Table.Columns(i).ColumnName
                boolCol.HeaderText = Table.Columns(i).ColumnName
                ts1.GridColumnStyles.Add(boolCol)
            Else
                Dim TextCol As New DataGridTextBoxColumnEx()
                TextCol.MappingName = Table.Columns(i).ColumnName
                TextCol.HeaderText = Table.Columns(i).ColumnName
                ts1.GridColumnStyles.Add(TextCol)
            End If
        Next
        Me.TableStyles.Add(ts1)
        Return ts1
    End Function

    Public Sub New()
        MyBase.New()
        InitializeComponent()
        objTimer.Interval = 1000
        objTimer.Enabled = False
        protNRowsInCaption = 3
        protRowHeaderWidth = Me.RowHeaderWidth
        CustomColumnHeaders = True
    End Sub

    Private Sub InitializeComponent()
        Me.components = New System.ComponentModel.Container()
        Me.objTimer = New System.Windows.Forms.Timer(Me.components)
    End Sub

    Private MouseNotif As Boolean = False

    Public Property MouseOverNotificationEnabled() As Boolean
        Get
            Return MouseNotif
        End Get
        Set(ByVal Value As Boolean)
            If Me.DesignMode = True Then
                Value = False
            End If
            MouseNotif = Value
            objTimer.Enabled = Value
        End Set
    End Property

    Private csea As New CellSelectedEventArgs(-1, -1)

    Private Overloads Sub objTimer_Elapsed(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles objTimer.Tick
        Dim myGrid As DataGrid = Me
        Dim r, c As Integer
        Dim p As Point = Me.PointToClient(Me.MousePosition)
        Dim myHitInfo As DataGrid.HitTestInfo = myGrid.HitTest(p.X, p.Y)
        If myHitInfo.Row >= 0 And myHitInfo.Column >= 0 And myHitInfo.Type = DataGrid.HitTestType.Cell Then
            r = myHitInfo.Row
            c = myHitInfo.Column
        Else
            r = -1
            c = -1
        End If
        If csea.rp <> r Or csea.cp <> c Then
            csea.rp = r
            csea.cp = c
            RaiseEvent MouseOverNotification(Me, csea)
        End If
    End Sub

    Private Sub DataGridEx_VisibleChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.VisibleChanged
        If Me.MouseNotif Then
            objTimer.Enabled = Me.Visible
        End If
    End Sub

    Private storedPageSettings As System.Drawing.Printing.PageSettings
    Public Property PageSettings() As System.Drawing.Printing.PageSettings
        Get
            Return storedPageSettings
        End Get
        Set(ByVal Value As System.Drawing.Printing.PageSettings)
            storedPageSettings = Value
        End Set
    End Property

    Public Sub PageSetup()

        Try
            Dim psDlg As New PageSetupDialog()

            If (storedPageSettings Is Nothing) Then
                storedPageSettings = New System.Drawing.Printing.PageSettings()
            End If

            psDlg.PageSettings = storedPageSettings
            psDlg.ShowDialog()
        Catch ex As Exception
            CustomControls.InformationMessageModule.InformationMessage("An error occurred - " & ex.Message)
        End Try

    End Sub

    Public Sub Print(ByVal objView As DataView, ByVal objTable As DataTable, ByVal cm As CurrencyManager)
        Try
            'Assumes the default printer
            Dim pd As CustomControls.DataGridPrintDocument = New CustomControls.DataGridPrintDocument(Me, objView, objTable, cm, -1, Nothing)

            If Not (storedPageSettings Is Nothing) Then
                pd.DefaultPageSettings = storedPageSettings
            End If


            Dim dlg As New PrintDialog()
            dlg.Document = pd
            Dim result As DialogResult = dlg.ShowDialog()

            If (result = DialogResult.OK) Then
                pd.Print()
            End If

        Catch ex As Exception
            InformationMessageModule.InformationMessage("An error occurred printing the DataGrid - " & ex.Message)
        End Try

    End Sub

    Public Sub PrintPreview(ByVal objView As DataView, ByVal objTable As DataTable, ByVal cm As CurrencyManager, ByVal NP As Integer, ByVal Question As String)

        Try
            'Assumes the default printer
            Dim pd As CustomControls.DataGridPrintDocument = New CustomControls.DataGridPrintDocument(Me, objView, objTable, cm, NP, Question)

            If Not (storedPageSettings Is Nothing) Then
                pd.DefaultPageSettings = storedPageSettings
            End If

            Dim dlg As New TablePrintPreviewDialog()
            dlg.TableDocument = pd
            dlg.ShowDialog()


        Catch ex As Exception
            InformationMessageModule.InformationMessage("An error occurred - " & ex.Message)
        End Try

    End Sub

    Public Property RowsInCaption() As Integer
        Get
            Return protNRowsInCaption
        End Get
        Set(ByVal Value As Integer)
            If Value <= 0 Then Return
            protNRowsInCaption = Value
        End Set
    End Property

    Public Property CustomColumnHeaders() As Boolean
        Get
            Return protCustomColumnsHeaders
        End Get
        Set(ByVal Value As Boolean)
            If Value Then
                If protCustomColumnsHeaders = False Then
                    protOldHeaderFont = Me.HeaderFont
                    Dim g As Graphics = Me.CreateGraphics()
                    Me.HeaderFont = New Font(System.Drawing.FontFamily.GenericSerif, (protNRowsInCaption) * Me.HeaderFont.GetHeight(g) - 8, FontStyle.Regular, GraphicsUnit.Point)
                    g.Dispose()
                    protRowHeaderWidth = Me.RowHeaderWidth
                End If
                Me.CaptionVisible = False
                Me.ColumnHeadersVisible = True
                'Me.RowHeaderWidth = 0
                'Me.RowHeadersVisible = False
            Else
                If Not Me.HeaderFont Is Nothing Then Me.HeaderFont.Dispose()
                Me.HeaderFont = Me.protOldHeaderFont
                Me.protOldHeaderFont = Nothing
                Me.CaptionVisible = True
                Me.ColumnHeadersVisible = True
                'Me.RowHeaderWidth = protRowHeaderWidth
                'Me.RowHeadersVisible = True
            End If
            protCustomColumnsHeaders = Value
        End Set
    End Property

    Protected Overloads Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
        If protCustomColumnsHeaders = True Then
            DrawHeader(e.Graphics, True, e.ClipRectangle, 0, 0)
            Return
        End If
        MyBase.OnPaint(e)
    End Sub

    Public Sub DrawHeader(ByVal g As Graphics, ByVal OffsetX As Integer, ByVal OffsetY As Integer)
        DrawHeader(g, False, Nothing, OffsetX, OffsetY)
    End Sub

    Public ReadOnly Property CustomColumnHeaderHeight() As Integer
        Get
            Return Me.HeaderFont.Height
        End Get
    End Property

    Public ReadOnly Property CustomColumnHeaderFont() As Font
        Get
            If protCustomColumnsHeaders Then
                Return Me.protOldHeaderFont
            End If
            Return Me.HeaderFont
        End Get
    End Property

    Private Sub DrawHeader(ByVal g As Graphics, ByVal CallNativePaint As Boolean, ByVal ClipRect As Rectangle, ByVal OffsetX As Integer, ByVal OffsetY As Integer)
        Dim rs As Rectangle
        Dim rs2 As Rectangle
        Dim rs3 As Rectangle
        If g Is Nothing Then
            g = Me.CreateGraphics()
        End If
        Dim ev As PaintEventArgs
        Dim top As Integer = Me.HeaderFont.Height
        If CallNativePaint Then
            ev = New PaintEventArgs(g, ClipRect)
            MyBase.OnPaint(ev)
        End If
        If ClipRect.Top < top Then
            Dim b As New SolidBrush(Me.HeaderBackColor)
            Dim b2 As New SolidBrush(Me.HeaderForeColor)
            Dim b3 As New SolidBrush(Me.BackgroundColor)
            Dim x As Integer = Me.RowHeaderWidth + 4
            Dim i As Integer
            Dim w, r As Integer
            Dim dx As Integer
            rs3 = New Rectangle(0, 2, x - 2, top + 6)
            For i = Me.FirstVisibleColumn To Me.FirstVisibleColumn + Me.VisibleColumnCount - 1
                If i = Me.FirstVisibleColumn Then
                    w = Me.GetCellBounds(Me.CurrentCell.RowNumber, i).Right - x - 1
                Else
                    w = Me.TableStyles(0).GridColumnStyles(i).Width
                End If
                r = w + 2

                If i = FirstVisibleColumn Then
                    dx = -4
                Else
                    dx = 0
                End If
                rs = New Rectangle(x + 2 + dx, 3, r - 3 - dx, top + 3)
                'rs2 = New Rectangle(x + 1 + dx, 1, r - dx, top + 8)
                If Me.FirstVisibleColumn + Me.VisibleColumnCount - 1 = i Then
                    rs2 = New Rectangle(x + 1 + dx, 2, r - dx - 1, top + 6)
                Else
                    rs2 = New Rectangle(x + 1 + dx, 2, r - dx, top + 6)
                End If
                g.FillRectangle(b, rs)
                ControlPaint.DrawBorder3D(g, rs2)
                PaintHeaderCell(g, Me.TableStyles(0).GridColumnStyles(i).HeaderText, Me.protOldHeaderFont, b2, rs, OffsetX, OffsetY)
                x += w
                If Me.FirstVisibleColumn + Me.VisibleColumnCount - 1 = i Then
                    Dim pn As New Drawing.Pen(Me.BackColor)
                    g.DrawLine(pn, x + 1, 2, x + 1, top + 6)
                    pn.Dispose()
                End If
            Next
            Dim p As New Drawing.Pen(Me.BackgroundColor)
            g.DrawLine(p, x + 2, 2, x + 2, top + 7)
            p.Dispose()
            b.Dispose()
            b2.Dispose()
            g.FillRectangle(b3, rs3)
            b3.Dispose()
            Return
        End If
    End Sub

    Private Sub DataGridEx_Scroll(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Scroll
        If protCustomColumnsHeaders = True Then
            Me.Invalidate()
        End If
    End Sub

    Public Sub PaintHeaderCell(ByVal g As Graphics, ByVal Sentence As String, ByVal F As Font, ByVal br As Brush, ByVal Bounds As Rectangle, ByVal offsetx As Integer, ByVal offsety As Integer)
        Dim strs(protNRowsInCaption - 1), tt() As String
        tt = Sentence.Split(" "c)
        Dim i, j, n As Integer
        j = 0
        Dim s As String = ""
        Dim os As String
        os = s
        For i = 0 To protNRowsInCaption - 1
            strs(i) = ""
        Next
        For i = 0 To tt.Length - 1
            If s <> "" Then
                s &= " " & tt(i)
            Else
                s = tt(i)
                os = s
            End If
            Dim w As Integer = CInt(g.MeasureString(s, F).Width)
            If w >= Bounds.Width Then
                strs(j) = os
                If s <> tt(i) Then
                    s = tt(i)
                    os = tt(i)
                Else
                    s = ""
                    os = ""
                End If
                If j = protNRowsInCaption - 1 And i <= tt.Length - 1 Then
                    strs(j) = "..."
                    Exit For
                End If
                j += 1
            Else
                os = s
            End If
            If i = tt.Length - 1 And j <> protNRowsInCaption Then
                strs(j) = s
            End If
        Next
        n = Me.protNRowsInCaption - 1
        Dim h As Integer = (Bounds.Height Mod F.Height) \ 2
        For i = 0 To n
            Dim w As Integer = CInt(g.MeasureString(strs(i), F).Width)
            If w < Bounds.Width Then
                w = (Bounds.Width - w) \ 2
            Else
                If Bounds.Width > 0 Then
                    While w >= Bounds.Width
                        strs(i) = strs(i).Substring(0, strs(i).Length - 1)
                        w = CInt(g.MeasureString(strs(i), F).Width)
                    End While
                Else
                    Return
                End If
                w = 0
            End If
            g.DrawString(strs(i), F, br, Bounds.X + w + offsetx, Bounds.Y + h + offsety)
            h += F.Height
        Next
    End Sub

    Public Function GetHeaderWidth(ByVal g As Graphics, ByVal Text As String) As Integer
        Dim strs() As String
        strs = Text.Split(" "c)
        Dim ws(strs.Length - 1) As Integer
        Dim i, n As Integer
        n = Math.Min(ws.Length - 1, Me.protNRowsInCaption - 1)
        Dim f As Font
        If protCustomColumnsHeaders Then
            f = protOldHeaderFont
        Else
            f = Me.HeaderFont
        End If
        For i = 0 To n
            ws(i) = CInt(g.MeasureString(strs(i), f).Width)
            GetHeaderWidth = Math.Max(GetHeaderWidth, ws(i))
        Next
    End Function
End Class

Public Class CellSelectedEventArgs
    Inherits System.EventArgs
    Friend rp, cp As Integer
    Public Sub New(ByVal r As Integer, ByVal c As Integer)
        rp = r
        cp = c
    End Sub
    Public ReadOnly Property Row() As Integer
        Get
            Return rp
        End Get
    End Property
    Public ReadOnly Property Column() As Integer
        Get
            Return cp
        End Get
    End Property
End Class
