Imports System.Drawing.Printing
Imports System.Reflection

Public Class DataGridPrintDocument
    Inherits PrintDocument

    Private printFont As Font
    Private headerFont As Font
    Private dataGridToPrint As CustomControls.DataGridEx
    Private objTable As DataTable
    Private objView As DataView
    Private privCM As CurrencyManager
    Private privNP As Integer
    Private privStrQuestion As String

    Public ReadOnly Property DataTable() As DataTable
        Get
            Return objTable
        End Get
    End Property

    Public Property NumberOfPagesPreviewed() As Integer
        Get
            Return privNP
        End Get
        Set(ByVal Value As Integer)
            privNP = Value
        End Set
    End Property

    Public Property QuestionAfterNPreviewedPages() As String
        Get
            Return privStrQuestion
        End Get
        Set(ByVal Value As String)
            privStrQuestion = Value
        End Set
    End Property

    Public Sub New(ByVal dataGridToPrint As CustomControls.DataGridEx, ByVal objView As DataView, ByVal objTable As DataTable, ByVal privCM As CurrencyManager, _
    ByVal NP As Integer, ByVal Question As String)
        MyBase.New()
        Me.dataGridToPrint = dataGridToPrint
        If objView Is Nothing Then
            Me.objView = objTable.DefaultView
        Else
            Me.objView = objView
        End If
        Me.objTable = objTable
        Me.privCM = privCM
        privNP = NP
        privStrQuestion = Question
    End Sub

    Private npp As Integer
    'Override OnBeginPrint to set up the font we are going to use
    Protected Overrides Sub OnBeginPrint(ByVal ev As PrintEventArgs)
        MyBase.OnBeginPrint(ev)
        CurRow = 0
        CurCol = 0
        NextFirstCol = 0
        npp = 0
        headerFont = dataGridToPrint.HeaderFont
        printFont = dataGridToPrint.DefaultFont
    End Sub

    Dim CurRow, CurCol As Integer
    Dim NextFirstCol As Integer
    'Override the OnPrintPage to provide the printing logic for the document
    Protected Overrides Sub OnPrintPage(ByVal ev As PrintPageEventArgs)

        MyBase.OnPrintPage(ev)

        NextFirstCol = 0
        npp += 1

        Dim lpp As Single = 0
        Dim yPos As Single = 0
        Dim count As Integer = 0
        Dim leftMargin As Single = ev.MarginBounds.Left
        Dim topMargin As Single = ev.MarginBounds.Top
        Dim line As String

        'Work out the number of lines per page
        'Use the MarginBounds on the event to do this
        lpp = ev.MarginBounds.Height / printFont.GetHeight(ev.Graphics)

        'Now iterate over the file printing out each line
        'NOTE WELL: This assumes that a single line is not wider than the page width
        'Check count first so that we don't read line that we won't print
        Dim tblStyle As DataGridTableStyle = dataGridToPrint.TableStyles(objTable.TableName)
        If tblStyle Is Nothing Then
            tblStyle = dataGridToPrint.CreateTableStyle(objTable)
        End If
        Dim rh As Integer = tblStyle.PreferredRowHeight
        Dim headerHeight As Integer
        If dataGridToPrint.CustomColumnHeaders = False Then
            headerHeight = rh
        Else
            headerHeight = dataGridToPrint.CustomColumnHeaderHeight
        End If
        Dim nrperpage As Integer = (ev.MarginBounds.Height \ rh) - (headerHeight \ rh)
        Dim i, j, w, l As Integer
        l = CInt(topMargin)
        w = CInt(leftMargin)
        Dim divPen As New Pen(dataGridToPrint.GridLineColor)
        Dim hfpen As New Pen(dataGridToPrint.HeaderForeColor)
        Dim hbBrush As New SolidBrush(dataGridToPrint.HeaderBackColor)
        Dim hfBrush As New SolidBrush(dataGridToPrint.HeaderForeColor)
        For j = CurCol To tblStyle.GridColumnStyles.Count - 1
            'Dim mi As MethodInfo = tblStyle.GridColumnStyles(j).GetType().GetMethod("Paint", BindingFlags.Public Or BindingFlags.NonPublic Or BindingFlags.Instance)
            'mi.GetParameters()
            If w + tblStyle.GridColumnStyles(j).Width >= ev.MarginBounds.Right And j <> CurCol Then
                NextFirstCol = j
                Exit For
            End If
            If tblStyle.GridColumnStyles(j).Width = 0 Then GoTo continuefor
            Dim r As New Rectangle(w, l, tblStyle.GridColumnStyles(j).Width, headerHeight)
            Dim rf As New Drawing.RectangleF(w, l, r.Width, r.Height)
            ev.Graphics.FillRectangle(hbBrush, r)
            'ev.Graphics.DrawRectangle(Pens.Black, r)

            If dataGridToPrint.CustomColumnHeaders = False Then
                ev.Graphics.DrawString(tblStyle.GridColumnStyles(j).HeaderText, headerFont, hfBrush, rf)
            Else
                dataGridToPrint.PaintHeaderCell(ev.Graphics, tblStyle.GridColumnStyles(j).HeaderText, _
                    dataGridToPrint.CustomColumnHeaderFont, hfBrush, r, 0, 0)
            End If
            w += tblStyle.GridColumnStyles(j).Width
continuefor:
        Next
        l += headerHeight
        For i = CurRow To nrperpage + CurRow - 1
            If i >= objView.Count Then Exit For
            w = CInt(leftMargin)
            For j = CurCol To tblStyle.GridColumnStyles.Count - 1
                'Dim mi As MethodInfo = tblStyle.GridColumnStyles(j).GetType().GetMethod("Paint", BindingFlags.Public Or BindingFlags.NonPublic Or BindingFlags.Instance)
                'mi.GetParameters()
                If w + tblStyle.GridColumnStyles(j).Width >= ev.MarginBounds.Right And j <> CurCol Then Exit For
                If tblStyle.GridColumnStyles(j).Width = 0 Then GoTo continuefor2
                Dim r As New Rectangle(w, l, tblStyle.GridColumnStyles(j).Width, rh)
                Dim rf As New Drawing.RectangleF(w, l, r.Width, r.Height)
                If TypeOf tblStyle.GridColumnStyles(j) Is IDataGridColumnConstraintEx Then
                    Dim tmp As IDataGridColumnConstraintEx = CType(tblStyle.GridColumnStyles(j), IDataGridColumnConstraintEx)
                    tmp.CellPaint(ev.Graphics, r, privCM, i, Brushes.White, Brushes.Black, False)
                    ev.Graphics.DrawRectangle(divPen, r)
                Else
                    ev.Graphics.DrawRectangle(divPen, r)
                    ev.Graphics.DrawString(objView.Item(i).Item(j).ToString(), printFont, Drawing.Brushes.Black, rf)
                End If
                w += tblStyle.GridColumnStyles(j).Width
continuefor2:
            Next
            l += rh
        Next

        l = CInt(topMargin)
        w = CInt(leftMargin)
        For j = CurCol To tblStyle.GridColumnStyles.Count - 1
            'Dim mi As MethodInfo = tblStyle.GridColumnStyles(j).GetType().GetMethod("Paint", BindingFlags.Public Or BindingFlags.NonPublic Or BindingFlags.Instance)
            'mi.GetParameters()
            If w + tblStyle.GridColumnStyles(j).Width >= ev.MarginBounds.Right And j <> CurCol Then
                Exit For
            End If
            If tblStyle.GridColumnStyles(j).Width = 0 Then GoTo continuefor3
            Dim r As New Rectangle(w, l, tblStyle.GridColumnStyles(j).Width, headerHeight)
            ev.Graphics.DrawRectangle(Pens.Black, r)

            w += tblStyle.GridColumnStyles(j).Width
continuefor3:
        Next

        CurRow = i

        If npp >= privNP Then
            If InformationMessageModule.QuestionMessage(privStrQuestion) = DialogResult.No Then
                ev.HasMorePages = False
                Return
            Else
                npp -= privNP
            End If
        End If
        'If we have more lines then print another page
        If (CurRow < objView.Count) Then
            ev.HasMorePages = True
        Else
            If NextFirstCol = 0 Then
                ev.HasMorePages = False
            Else
                CurRow = 0
                CurCol = NextFirstCol
                ev.HasMorePages = True
            End If
        End If
        divPen.Dispose()
        hfpen.Dispose()
        hbBrush.Dispose()
        hfBrush.Dispose()
    End Sub
End Class
