Option Strict On
Option Explicit On 

Imports System.Drawing
Imports System.Runtime.Serialization
Imports System.ComponentModel

<Serializable()> _
Public MustInherit Class ColumnDefinition
    Implements ISerializable

    Protected privName As String
    Protected privType As System.Type
    Protected privHidden As Integer
    Protected privColor As Color
    Protected privHeaderFunction As CreateHeaderText
    Protected privPreferredWidth As Integer

    'Deserialization constructor.
    Public Sub New(ByVal info As SerializationInfo, ByVal context As StreamingContext)
        Try
            privName = CStr(info.GetValue("privName", GetType(String)))
            'privType = Type.GetType(CStr(info.GetValue("privType", GetType(System.String))))
            privType = System.Type.GetType(CStr(info.GetValue("privType", GetType(System.String))))
            privHidden = info.GetInt32("privHidden")
            privPreferredWidth = info.GetInt32("privPreferredWidth")
            privColor = Color.FromArgb(CInt(info.GetValue("privColor", GetType(Integer))))
            privHeaderFunction = Nothing 'AddressOf DetaultHeaderFunction
        Catch ex As Exception
#If DebugTrace Then
            Dim dumper1 As ObjectDumper.FileDumper
            If TypeOf Me Is FixedColumnDefinition Then
                dumper1 = New ObjectDumper.FileDumper("c:\log2.txt")
            Else
                dumper1 = New ObjectDumper.FileDumper("c:\log.txt")
            End If
            dumper1.Dump("Deserialization Exception: " & ex.Message)
#End If
        End Try
    End Sub 'New

    'Private Function DetaultHeaderFunction(ByVal InitialText As String, ByVal ColumnName As String) As String
    '    Return Nothing
    'End Function

    'Serialization function.
    Public Overridable Sub GetObjectData(ByVal info As SerializationInfo, ByVal context As StreamingContext) Implements ISerializable.GetObjectData
        info.AddValue("privName", privName)
        info.AddValue("privType", privType.FullName)
        info.AddValue("privHidden", CInt(privHidden))
        info.AddValue("privPreferredWidth", CInt(privPreferredWidth))
        info.AddValue("privColor", privColor.ToArgb())
    End Sub 'GetObjectData

    Public Sub New(ByVal Name As String, ByVal Type As System.Type, ByVal Hidden As Boolean, ByVal HeaderCreationFunction As CreateHeaderText, ByVal PreferredWidth As Integer)
        privName = Name
        privType = Type
        If Hidden Then privHidden = -1 Else privHidden = 0
        privColor = Color.White
        privHeaderFunction = HeaderCreationFunction
        Me.privPreferredWidth = PreferredWidth
    End Sub

    Public Sub New()
        Dim rnd As New Random(DateTime.Now.Millisecond)
        privName = "ColumnName" & rnd.Next(0, Int32.MaxValue)
        privType = GetType(System.String)
        privHidden = 0
        privPreferredWidth = -1
        privColor = Color.White
        privHeaderFunction = Nothing 'AddressOf DetaultHeaderFunction
    End Sub

    Public Sub New(ByVal obj As Object)
        privName = CStr(ObjectDumper.Dumper.GetFieldValue(obj, "privName"))
        privType = CType(ObjectDumper.Dumper.GetFieldValue(obj, "privType"), System.Type)
        privHidden = CType(ObjectDumper.Dumper.GetFieldValue(obj, "privHidden"), System.Int32)
        privColor = CType(ObjectDumper.Dumper.GetFieldValue(obj, "privColor"), System.Drawing.Color)
        'privColor = Color.White
        privHeaderFunction = Nothing 'AddressOf DetaultHeaderFunction 'CType(ObjectDumper.Dumper.GetFieldValue(obj, "privHeaderFunction"), CreateHeaderText)
    End Sub

    Public Property ColumnName() As String
        Get
            Return privName
        End Get
        Set(ByVal Value As String)
            privName = Value
        End Set
    End Property

    Public Property Type() As System.Type
        Get
            Return privType
        End Get
        Set(ByVal Value As System.Type)
            privType = Value
        End Set
    End Property

    Public Property Hidden() As Boolean
        Get
            If privHidden <> 0 Then
                Return True
            Else
                Return False
            End If
        End Get
        Set(ByVal Value As Boolean)
            If Value Then
                privHidden = -1
            Else
                privHidden = 0
            End If
        End Set
    End Property

    Public Property Color() As Color
        Get
            Return privColor
        End Get
        Set(ByVal value As Color)
            privColor = value
        End Set
    End Property

    Public Property PreferredWidth() As Integer
        Get
            Return privPreferredWidth
        End Get
        Set(ByVal Value As Integer)
            privPreferredWidth = Value
        End Set
    End Property

    Public Delegate Function CreateHeaderText(ByVal InitialText As String, ByVal ColumnName As String) As String

    Public Function HeaderText(ByVal InitialText As String) As String
        If privHeaderFunction Is Nothing Then
            If InitialText Is Nothing Then
                Return Me.privName
            Else
                Return InitialText
            End If
        Else
            Return privHeaderFunction.Invoke(InitialText, Me.privName)
        End If
    End Function

    Public Function HeaderText() As String
        Return HeaderText(Nothing)
    End Function
End Class

<Serializable(), TypeConverter(GetType(ItemColumnDefinitionConverter))> _
Public Class ItemColumnDefinition
    Inherits ColumnDefinition
    Protected privAllowDbNull As Boolean
    Protected privReadOnly As Boolean
    Protected privDefaultValue As Object

    'Deserialization constructor.
    Public Sub New(ByVal info As SerializationInfo, ByVal context As StreamingContext)
        MyBase.New(info, context)
        Try
            privAllowDbNull = CBool(info.GetValue("privAllowDbNull", GetType(Int32)))
            privReadOnly = CBool(info.GetValue("privReadOnly", GetType(Int32)))
            Dim privTypeName As String = CStr(info.GetValue("privTypeName", GetType(System.String)))
            privDefaultValue = info.GetValue("privDefaultValue", System.Type.GetType(privTypeName))
        Catch ex As Exception
#If DebugTrace Then
            Dim dumper1 As New ObjectDumper.FileDumper("c:\log.txt")
            dumper1.Dump("Deserialization Exception: " & ex.Message)
#End If
        End Try
    End Sub 'New

    'Serialization function.
    Public Overrides Sub GetObjectData(ByVal info As SerializationInfo, ByVal context As StreamingContext)
        info.FullTypeName = "GanttLib.ItemColumnDefinition"
        info.AssemblyName = "GanttLib"
        MyBase.GetObjectData(info, context)
        info.AddValue("privAllowDbNull", CInt(privAllowDbNull))
        info.AddValue("privReadOnly", CInt(privReadOnly))
        info.AddValue("privTypeName", privDefaultValue.GetType.FullName)
        info.AddValue("privDefaultValue", privDefaultValue)
    End Sub 'GetObjectData

    Public Sub New(ByVal Name As String, ByVal Type As System.Type, _
            ByVal Hidden As Boolean, ByVal HeaderCreationFunction As CreateHeaderText, ByVal AllowDbNull As Boolean, _
            ByVal DefaultValue As Object, ByVal PreferredWidth As Integer, ByVal IsReadOnly As Boolean)
        MyBase.New(Name, Type, Hidden, HeaderCreationFunction, PreferredWidth)
        privAllowDbNull = AllowDbNull
        privDefaultValue = DefaultValue
        privReadOnly = IsReadOnly
    End Sub

    Public Sub New()
        MyBase.New()
        privType = GetType(System.Boolean)
        privDefaultValue = False
        privAllowDbNull = True
        privReadOnly = False
    End Sub

    Public Sub New(ByVal obj As Object)
        MyBase.New(obj)
        privDefaultValue = ObjectDumper.Dumper.GetFieldValue(obj, "privDefaultValue")
        privAllowDbNull = CType(ObjectDumper.Dumper.GetFieldValue(obj, "privAllowDbNull"), System.Boolean)
        privReadOnly = CType(ObjectDumper.Dumper.GetFieldValue(obj, "privReadOnly"), System.Boolean)
    End Sub

    Public Property AllowDbNull() As Boolean
        Get
            Return privAllowDbNull
        End Get
        Set(ByVal Value As Boolean)
            privAllowDbNull = Value
        End Set
    End Property

    Public Property [ReadOnly]() As Boolean
        Get
            Return privReadOnly
        End Get
        Set(ByVal Value As Boolean)
            privReadOnly = Value
        End Set
    End Property

    Public Property DefaultValue() As Object
        Get
            Return privDefaultValue
        End Get
        Set(ByVal Value As Object)
            privDefaultValue = Value
        End Set
    End Property
End Class

<Serializable(), TypeConverter(GetType(FixedColumnDefinitionConverter))> _
Public Class FixedColumnDefinition
    Inherits ColumnDefinition

    Protected privNumberOfElements As Integer
    Protected privInitialValue As Object
    Protected privEvaluateFunction As EvaluateValue
    Protected privInitObject As Object
    Protected privIsTimeColumn As Integer

    'Deserialization constructor.
    Public Sub New(ByVal info As SerializationInfo, ByVal context As StreamingContext)
        MyBase.New(info, context)
        Try
            privNumberOfElements = info.GetInt32("privNumberOfElements")
            Dim privTypeName As String = info.GetString("privTypeName")
            privInitialValue = info.GetValue("privInitialValue", System.Type.GetType(privTypeName))
            Dim t As Integer = info.GetInt32("privIsTimeColumn")
            If t <> 0 Then privIsTimeColumn = -1 Else privIsTimeColumn = 0
            privIsTimeColumn = -1
            Me.privEvaluateFunction = AddressOf DefaultEvaluate
            Me.privInitObject = New Object()
        Catch ex As Exception
#If DebugTrace Then
            Dim dumper1 As New ObjectDumper.FileDumper("c:\log2.txt")
            dumper1.Dump("Deserialization Exception: " & ex.Message)
#End If
        End Try
    End Sub 'New

    'Serialization function.
    Public Overrides Sub GetObjectData(ByVal info As SerializationInfo, ByVal context As StreamingContext)
        info.FullTypeName = "GanttLib.FixedColumnDefinition"
        info.AssemblyName = "GanttLib"
        MyBase.GetObjectData(info, context)
        info.AddValue("privNumberOfElements", privNumberOfElements)
        info.AddValue("privTypeName", privInitialValue.GetType().FullName)
        info.AddValue("privInitialValue", privInitialValue)
        info.AddValue("privIsTimeColumn", privIsTimeColumn)
    End Sub 'GetObjectData

    Public Sub New(ByVal Name As String, ByVal Type As System.Type, ByVal Hidden As Boolean, _
                    ByVal HeaderCreationFunction As CreateHeaderText, _
                    ByVal NumberOfElements As Integer, _
                    ByVal IsTimeColumn As Boolean, _
                    ByVal InitialValue As Object, _
                    ByVal EvaluateFunction As EvaluateValue, _
                    ByVal InitObject As Object, _
                    ByVal PreferredWidth As Integer)
        MyBase.New(Name, Type, Hidden, HeaderCreationFunction, PreferredWidth)
        Me.privNumberOfElements = NumberOfElements
        Me.privInitialValue = InitialValue
        Me.privEvaluateFunction = AddressOf DefaultEvaluate
        If Not EvaluateFunction Is Nothing Then Me.privEvaluateFunction = EvaluateFunction
        Me.privInitObject = InitObject
        If IsTimeColumn Then Me.privIsTimeColumn = -1 Else Me.privIsTimeColumn = 0
    End Sub

    Public Sub New()
        MyBase.New()
        Me.privType = GetType(System.Int32)
        Me.privNumberOfElements = 10
        Me.privInitialValue = 1
        Me.privEvaluateFunction = AddressOf DefaultEvaluate
        Me.privInitObject = New Object()
        Me.privIsTimeColumn = -1
    End Sub

    Public Sub New(ByVal obj As Object)
        MyBase.New(obj)
        Me.privNumberOfElements = CType(ObjectDumper.Dumper.GetFieldValue(obj, "privNumberOfElements"), System.Int32)
        Me.privInitialValue = ObjectDumper.Dumper.GetFieldValue(obj, "privInitialValue")
        If Me.privInitialValue Is Nothing Then Me.privInitialValue = 1
        Me.privEvaluateFunction = AddressOf DefaultEvaluate
        'Me.privEvaluateFunction = CType(ObjectDumper.Dumper.GetFieldValue(obj, "privEvaluateFunction"), EvaluateValue)
        Me.privInitObject = CType(ObjectDumper.Dumper.GetFieldValue(obj, "privInitObject"), System.Object)
        If Me.privInitObject Is Nothing Then Me.privInitObject = New Object()
        Me.privIsTimeColumn = CType(ObjectDumper.Dumper.GetFieldValue(obj, "privIsTimeColumn"), System.Int32)
    End Sub

    Private Function DefaultEvaluate(ByVal Initial As Object, _
            ByVal InitObject As Object, ByVal objGantt As GanttData, ByVal i As Integer, ByVal curRow As DataRow) As Object
        If TypeOf (Initial) Is DateTime Then Return CType(Initial, DateTime).AddDays(i).ToShortDateString()
        Return CInt(Initial) + i
    End Function

    Public Property NumberOfElements() As Integer
        Get
            Return privNumberOfElements
        End Get
        Set(ByVal Value As Integer)
            If Value > 0 Then
                privNumberOfElements = Value
            End If
        End Set
    End Property

    Public Property InitialValue() As Object
        Get
            Return privInitialValue
        End Get
        Set(ByVal Value As Object)
            privInitialValue = Value
        End Set
    End Property

    Public Property EvaluateFunction() As EvaluateValue
        Get
            Return privEvaluateFunction
        End Get
        Set(ByVal Value As EvaluateValue)
            privEvaluateFunction = Value
        End Set
    End Property

    Public Property InitObject() As Object
        Get
            Return privInitObject
        End Get
        Set(ByVal Value As Object)
            privInitObject = Value
        End Set
    End Property

    Public Property IsTimeColumn() As Boolean
        Get
            If privIsTimeColumn <> 0 Then
                Return True
            Else
                Return False
            End If
        End Get
        Set(ByVal Value As Boolean)
            If Value Then
                privIsTimeColumn = -1
            Else
                privIsTimeColumn = 0
            End If
        End Set
    End Property

    Public Delegate Function EvaluateValue(ByVal Initial As Object, _
            ByVal InitObject As Object, ByVal objGantt As GanttData, ByVal i As Integer, ByVal curRow As DataRow) As Object
End Class