Option Strict On
Option Explicit On 

Imports System.ComponentModel

<Serializable()> _
Public Class GanttData
    Inherits System.ComponentModel.Component

    Private WithEvents tGanttTable As DataTable
    Private WithEvents myDataSet As DataSet
    Private privDataSetName As String
    Private privTableName As String
    Private privFixedFields As FixedColumnDefinitionList
    Private privItemFields As ItemColumnDefinitionList
    Private privItemNames() As String
    Private privModifiedFlag As Boolean

#If DebugTrace Then
    Private dumper1 As New ObjectDumper.FileDumper("c:\log1.txt")
    Private dumper2 As New ObjectDumper.FileDumper("c:\log2.txt")
#End If

    Public Sub New(ByVal FixedFields As FixedColumnDefinitionList, _
                    ByVal ItemFields As ItemColumnDefinitionList, _
                    ByVal ItemNames() As String, _
                    ByVal DataSetName As String, ByVal TableName As String)
        Me.privDataSetName = DataSetName
        Me.privTableName = TableName
        Me.privFixedFields = FixedFields
        Me.privItemFields = ItemFields
        Me.privItemNames = ItemNames
        Me.privModifiedFlag = False
        MakeDataSet()
    End Sub

    Public Sub New()
        MyBase.New()
        privDataSetName = "GanttDataSet"
        privTableName = "GanttTable"
        privFixedFields = New FixedColumnDefinitionList()
        privItemFields = New ItemColumnDefinitionList()
    End Sub

    Public Property FixedFieldDefinitions() As FixedColumnDefinition()
        Get
            Dim retArray(-1) As FixedColumnDefinition
            If privFixedFields.Count = 0 Then Return retArray
            ReDim retArray(privFixedFields.Count - 1)
            Dim i As Integer
            For i = 0 To privFixedFields.Count - 1
                retArray(i) = privFixedFields(i)
            Next
            Return retArray
        End Get
        Set(ByVal value As FixedColumnDefinition())
            If value Is Nothing Then Return
            Dim e As IEnumerator
            e = value.GetEnumerator
            privFixedFields.Clear()
            Dim i As Integer
            For i = 0 To value.GetLength(0) - 1
                e.MoveNext()
                If Not e.Current Is Nothing Then
                    privFixedFields.Add(CType(e.Current, FixedColumnDefinition))
                End If
            Next
            MakeDataSet()
        End Set
    End Property

    Public Property ItemFieldDefinitions() As ItemColumnDefinition()
        Get
            Dim retArray(-1) As ItemColumnDefinition
            If privItemFields.Count = 0 Then Return retArray
            ReDim retArray(privItemFields.Count - 1)
            Dim i As Integer
            For i = 0 To privItemFields.Count - 1
                retArray(i) = privItemFields(i)
            Next
            Return retArray
        End Get
        Set(ByVal value As ItemColumnDefinition())
            If value Is Nothing Then Return
            Dim e As IEnumerator
            e = value.GetEnumerator
            privItemFields.Clear()
            Dim i As Integer
            For i = 0 To value.GetLength(0) - 1
                e.MoveNext()
                If Not e.Current Is Nothing Then
                    privItemFields.Add(CType(e.Current, ItemColumnDefinition))
                End If
            Next
            MakeDataSet()
        End Set
    End Property

    Public Property ItemNames() As String()
        Get
            Return privItemNames
        End Get
        Set(ByVal Value As String())
            privItemNames = Value
            MakeDataSet()
        End Set
    End Property

    Public Event DataSetReady As System.EventHandler

    Public Function CheckForGanttInit() As Boolean
        Try
            If privItemNames Is Nothing Or _
                privItemFields.Count = 0 Or _
                privFixedFields.Count = 0 Then
                Return False
            End If
            If (privItemNames.GetLength(0) = 1 And privItemNames(0) Is Nothing) Or _
                (privItemFields.GetLength(0) = 1 And privItemFields(0) Is Nothing) Or _
                (privFixedFields.GetLength(0) = 1 And privFixedFields(0) Is Nothing) Then
                Return False
            End If
            Return True
        Catch ex As Exception
            Return False
        End Try
    End Function

    Private Sub MakeDataSet()
        If privItemNames Is Nothing Then
            ReDim privItemNames(-1)
        End If

        ' Create a DataSet.
        myDataSet = New DataSet(privDataSetName)

        ' Create the DataTable.
        tGanttTable = New DataTable(privTableName)
        tGanttTable.DefaultView.AllowNew = False
        tGanttTable.DefaultView.AllowDelete = False

        ' Create fixed columns, and add them to the table.
        Dim i As Integer
        Dim cCol As DataColumn
        For i = 0 To privFixedFields.GetLength(0) - 1
            cCol = New DataColumn(privFixedFields(i).ColumnName, privFixedFields(i).Type)
            cCol.AllowDBNull = False
            tGanttTable.Columns.Add(cCol)
        Next

        Dim j As Integer
        For j = 1 To privItemNames.GetLength(0)
            For i = 0 To privItemFields.GetLength(0) - 1
                cCol = New DataColumn(privItemFields(i).ColumnName & j, privItemFields(i).Type)
                cCol.AllowDBNull = privItemFields(i).AllowDbNull
                cCol.DefaultValue = privItemFields(i).DefaultValue
                tGanttTable.Columns.Add(cCol)
            Next
        Next

        ' Add the tables to the DataSet.
        myDataSet.Tables.Add(tGanttTable)

        'Initialize Gantt Fixed Columns
        Dim vals(Me.privFixedFields.GetLength(0) - 1) As Object
        InitRows(vals, 0, Nothing)

        ' DataSet is Ready to Use by other components
        'If Me.DesignMode = False Then
        Me.privModifiedFlag = False
        RaiseEvent DataSetReady(Me, New System.EventArgs())
        'End If
    End Sub

    Public Function GetItemColumnName(ByVal ItemName As String, ByVal ItemFieldNumber As Integer) As String
        Return ItemName & (ItemFieldNumber + 1)
    End Function


    Private Sub InitRows(ByVal ColumnValues() As Object, ByVal CurColumn As Integer, _
                        ByVal curRow As DataRow)
        Dim i, j As Integer
        If CurColumn >= privFixedFields.GetLength(0) Then
            If Not curRow Is Nothing Then tGanttTable.Rows.Add(curRow)
            Return
        End If
        If Not privFixedFields(CurColumn).IsTimeColumn Then Return
        Dim GanttDef As FixedColumnDefinition = privFixedFields(CurColumn)
        For i = 0 To GanttDef.NumberOfElements - 1
            If i <> 0 Then curRow = Nothing
            If curRow Is Nothing Then
                curRow = tGanttTable.NewRow()
                For j = 0 To CurColumn - 1
                    curRow(j) = ColumnValues(j)
                Next
            End If
            ColumnValues(CurColumn) = GanttDef.EvaluateFunction.Invoke(GanttDef.InitialValue, GanttDef.InitObject, Me, i, curRow)
            curRow(CurColumn) = ColumnValues(CurColumn)
            InitRows(ColumnValues, CurColumn + 1, curRow)
        Next
    End Sub

    Public ReadOnly Property NumberOfFieldsForItem() As Int32
        Get
            Return privItemFields.GetLength(0)
        End Get
    End Property

    Public ReadOnly Property NumberOfFixedFields() As Integer
        Get
            Return privFixedFields.GetLength(0)
        End Get
    End Property

    Public ReadOnly Property NumberOfItems() As Integer
        Get
            If privItemNames Is Nothing Then Return 0
            Return privItemNames.GetLength(0)
        End Get
    End Property

    Public ReadOnly Property ItemName(ByVal ItemNumber As Integer) As String
        Get
            If ItemNumber < 0 Or ItemNumber >= privItemNames.GetLength(0) Then
                Return Nothing
            End If
            Return privItemNames(ItemNumber)
        End Get
    End Property

    Public ReadOnly Property FieldName(ByVal ItemNumber As Integer, ByVal FieldNumber As Integer) As String
        Get
            If ItemNumber < 0 Or ItemNumber >= privItemNames.GetLength(0) _
                Or FieldNumber < 0 Or FieldNumber >= privItemFields.GetLength(0) Then
                Return Nothing
            End If
            Return privItemFields(FieldNumber).ColumnName & (ItemNumber + 1)
        End Get
    End Property

    <Browsable(False)> _
    Public ReadOnly Property DataSet() As DataSet
        Get
            Return myDataSet
        End Get
    End Property

    <Browsable(False)> _
    Public ReadOnly Property DataTable() As DataTable
        Get
            If myDataSet Is Nothing Then Return Nothing
            Return myDataSet.Tables(0)
        End Get
    End Property

    Public Property ModifiedFlag() As Boolean
        Get
            Return privModifiedFlag
        End Get
        Set(ByVal Value As Boolean)
            privModifiedFlag = Value
            If Value = False And Not myDataSet Is Nothing Then myDataSet.AcceptChanges()
        End Set
    End Property

    Public Property FieldValue(ByVal ItemNumber As Integer, ByVal FieldName As String, ByVal Row As Integer) As Object
        Get
            Return tGanttTable.Rows(Row).Item(EvaluateColumnIndex(ItemNumber, FieldName))
        End Get
        Set(ByVal Value As Object)
            Dim oldvalue As Object = tGanttTable.Rows(Row).Item(EvaluateColumnIndex(ItemNumber, FieldName))
            If oldvalue.Equals(Value) = False Then
                tGanttTable.Rows(Row).Item(EvaluateColumnIndex(ItemNumber, FieldName)) = Value
                privModifiedFlag = True
            End If
        End Set
    End Property

    Public Property FieldValue(ByVal ItemNumber As Integer, ByVal FieldName As String, ByVal tRow As DataRow) As Object
        Get
            Return tRow.Item(EvaluateColumnIndex(ItemNumber, FieldName))
        End Get
        Set(ByVal Value As Object)
            Dim oldvalue As Object = tRow.Item(EvaluateColumnIndex(ItemNumber, FieldName))
            If oldvalue.Equals(Value) = False Then
                tRow.Item(EvaluateColumnIndex(ItemNumber, FieldName)) = Value
                privModifiedFlag = True
            End If
        End Set
    End Property

    Public Property FixedFieldValue(ByVal FieldName As String, ByVal Row As Integer) As Object
        Get
            Return tGanttTable.Rows(Row).Item(EvaluateFixedColumnIndex(FieldName))
        End Get
        Set(ByVal Value As Object)
            Dim oldvalue As Object = tGanttTable.Rows(Row).Item(EvaluateFixedColumnIndex(FieldName))
            If oldvalue.Equals(Value) = False Then
                tGanttTable.Rows(Row).Item(EvaluateFixedColumnIndex(FieldName)) = Value
                privModifiedFlag = True
            End If
        End Set
    End Property

    Public Property FixedFieldValue(ByVal FieldName As String, ByVal tRow As DataRow) As Object
        Get
            Return tRow.Item(EvaluateFixedColumnIndex(FieldName))
        End Get
        Set(ByVal Value As Object)
            Dim oldvalue As Object = tRow.Item(EvaluateFixedColumnIndex(FieldName))
            If oldvalue.Equals(Value) = False Then
                tRow.Item(EvaluateFixedColumnIndex(FieldName)) = Value
                privModifiedFlag = True
            End If
        End Set
    End Property

    Private Function EvaluateColumnIndex(ByVal ItemNumber As Integer, ByVal FieldName As String) As Integer
        Dim ret As Integer
        ret = Me.NumberOfFixedFields
        Dim i As Integer
        For i = 0 To Me.privItemFields.GetLength(0) - 1
            If Me.privItemFields(i).ColumnName = FieldName Then Exit For
        Next
        If i >= Me.privItemFields.GetLength(0) Then Return -1

        Return ret + ItemNumber * Me.NumberOfFieldsForItem + i
    End Function

    Private Function EvaluateFixedColumnIndex(ByVal FieldName As String) As Integer
        Dim i As Integer
        For i = 0 To Me.privFixedFields.GetLength(0) - 1
            If Me.privFixedFields(i).ColumnName = FieldName Then Exit For
        Next
        If i >= Me.privFixedFields.GetLength(0) Then Return -1

        Return i
    End Function

    Public Function GetItemType(ByVal col As Integer) As String
        col -= Me.NumberOfFixedFields
        If col < 0 Then Return Nothing
        Return privItemFields(col Mod NumberOfFieldsForItem).ColumnName
    End Function

    Public Function GetItemNumber(ByVal col As Integer) As Integer
        col -= Me.NumberOfFixedFields
        If col < 0 Then Return -1
        Return col \ NumberOfFieldsForItem
    End Function

End Class
