Imports System.Windows.Forms
Imports System.Drawing
Imports System.ComponentModel

Public Class GanttGrid
    Inherits CustomControls.DataGridEx

    Public Sub New()
        MyBase.New()

        Me.AllowDrop = True
        Me.MouseOverNotificationEnabled = True
        Me.SelectionMode = CustomControls.GridSelection.Cell

    End Sub


    Private privGantt As GanttData

    Public Sub New(ByVal GanttDefinition As GanttData)
        Me.New()
        privGantt = GanttDefinition
    End Sub

    Public Property GanttData() As GanttData
        Get
            Return privGantt
        End Get
        Set(ByVal Value As GanttData)
            privGantt = Value
            If Value Is Nothing Then Return
            AddHandler privGantt.DataSetReady, AddressOf Me.MakeTableStyles
            If privGantt.CheckForGanttInit() Then
                AddCustomDataTableStyle()
            End If
        End Set
    End Property

    Private DefaultWidth As Integer = 50

    Public Property ItemWidth() As Integer
        Get
            Return DefaultWidth
        End Get
        Set(ByVal Value As Integer)
            DefaultWidth = Value
        End Set
    End Property

    Public Function GetTable() As DataTable
        Return privGantt.DataTable
    End Function

    Private Sub AddCustomDataTableStyle()
        If privGantt Is Nothing Then Return
        If privGantt.DataSet Is Nothing Then Return
        If privGantt.DataTable Is Nothing Then Return
        Me.TableStyles.Clear()
        Dim ts1 As New DataGridTableStyle()
        Dim boolCol As CustomControls.DataGridBoolColumnEx
        Dim dataCol As CustomControls.DataGridTextBoxColumnEx
        ts1.MappingName = privGantt.DataSet.Tables(0).TableName
        ts1.AlternatingBackColor = Color.LightGray
        ' Set other properties.

        Dim i, j As Integer
        For i = 0 To privGantt.NumberOfFixedFields - 1
            If privGantt.FixedFieldDefinitions(i).Type Is GetType(System.Boolean) Then
                boolCol = New CustomControls.DataGridBoolColumnEx(privGantt.FixedFieldDefinitions(i).Color)
                boolCol.MappingName = privGantt.FixedFieldDefinitions(i).ColumnName
                boolCol.HeaderText = privGantt.FixedFieldDefinitions(i).HeaderText()
                boolCol.ReadOnly = True
                If privGantt.FixedFieldDefinitions(i).Hidden Then boolCol.Width = 0
                If privGantt.FixedFieldDefinitions(i).PreferredWidth >= 0 Then boolCol.Width = privGantt.FixedFieldDefinitions(i).PreferredWidth
                ts1.GridColumnStyles.Add(boolCol)
            Else
                dataCol = New CustomControls.DataGridTextBoxColumnEx()
                dataCol.MappingName = privGantt.FixedFieldDefinitions(i).ColumnName
                dataCol.HeaderText = privGantt.FixedFieldDefinitions(i).HeaderText()
                dataCol.ReadOnly = True
                If privGantt.FixedFieldDefinitions(i).Hidden Then dataCol.Width = 0
                If privGantt.FixedFieldDefinitions(i).PreferredWidth >= 0 Then dataCol.Width = privGantt.FixedFieldDefinitions(i).PreferredWidth
                ts1.GridColumnStyles.Add(dataCol)
            End If
        Next
        For j = 1 To privGantt.NumberOfItems
            For i = 0 To privGantt.NumberOfFieldsForItem - 1
                If privGantt.ItemFieldDefinitions(i).Type Is GetType(System.Boolean) Then
                    boolCol = New CustomControls.DataGridBoolColumnEx(privGantt.ItemFieldDefinitions(i).Color)
                    boolCol.MappingName = privGantt.FieldName(j - 1, i)
                    boolCol.HeaderText = privGantt.ItemFieldDefinitions(i).HeaderText(privGantt.ItemName(j - 1) & " - " & privGantt.ItemFieldDefinitions(i).ColumnName)
                    boolCol.ReadOnly = privGantt.ItemFieldDefinitions(i).ReadOnly
                    If privGantt.ItemFieldDefinitions(i).Hidden Then
                        boolCol.Width = 0
                    Else
                        If privGantt.ItemFieldDefinitions(i).PreferredWidth >= 0 Then
                            boolCol.Width = privGantt.ItemFieldDefinitions(i).PreferredWidth
                        Else
                            boolCol.Width = DefaultWidth
                        End If
                    End If
                    ts1.GridColumnStyles.Add(boolCol)
                Else
                    dataCol = New CustomControls.DataGridTextBoxColumnEx()
                    dataCol.MappingName = privGantt.FieldName(j - 1, i)
                    dataCol.HeaderText = privGantt.ItemFieldDefinitions(i).HeaderText(privGantt.ItemName(j - 1) & " - " & privGantt.ItemFieldDefinitions(i).ColumnName)
                    dataCol.ReadOnly = privGantt.ItemFieldDefinitions(i).ReadOnly
                    If privGantt.ItemFieldDefinitions(i).Hidden Then
                        dataCol.Width = 0
                    Else
                        If privGantt.ItemFieldDefinitions(i).PreferredWidth >= 0 Then
                            dataCol.Width = privGantt.ItemFieldDefinitions(i).PreferredWidth
                        Else
                            dataCol.Width = DefaultWidth
                        End If
                    End If
                    ts1.GridColumnStyles.Add(dataCol)
                End If
            Next
        Next

        Me.TableStyles.Add(ts1)
        Dim dv As New DataView(privGantt.DataTable)
        dv.AllowDelete = False
        dv.AllowNew = False
        Me.SetDataBinding(dv, Nothing)
        RaiseEvent CustomTableStyleAvailable(Me, New System.EventArgs())
    End Sub

    Public Event CustomTableStyleAvailable As System.EventHandler

    <Browsable(False)> _
    Public Property ItemVisible(ByVal ItemNumber As Integer) As Boolean
        Get
            Dim i As Integer
            If privGantt Is Nothing Then Return True
            For i = 0 To privGantt.NumberOfFieldsForItem - 1
                If privGantt.ItemFieldDefinitions(i).Hidden = False Then
                    If Me.TableStyles(0).GridColumnStyles(privGantt.NumberOfFixedFields + privGantt.NumberOfFieldsForItem * ItemNumber + i).Width <> 0 Then
                        Return True
                    End If
                End If
            Next
            Return False
        End Get
        Set(ByVal Value As Boolean)
            Dim w As Integer
            If Value Then
                w = DefaultWidth
            Else
                w = 0
            End If
            Dim i As Integer
            If privGantt Is Nothing Then Return
            For i = 0 To privGantt.NumberOfFieldsForItem - 1
                If privGantt.ItemFieldDefinitions(i).Hidden = False Then
                    Me.TableStyles(0).GridColumnStyles(privGantt.NumberOfFixedFields + privGantt.NumberOfFieldsForItem * ItemNumber + i).Width = w
                Else
                    Me.TableStyles(0).GridColumnStyles(privGantt.NumberOfFixedFields + privGantt.NumberOfFieldsForItem * ItemNumber + i).Width = 0
                End If
            Next
        End Set
    End Property


    Public Sub SetToolTip(ByVal objToolTip As ToolTip, ByVal Description As String)
        objToolTip.SetToolTip(Me, Description)
    End Sub

    Private DragPar As New DragDropPars()

    Private Sub objDataGridEx_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles MyBase.MouseDown
        If (e.Button And MouseButtons.Right) <> MouseButtons.Right Then Return
        Dim r, c As Integer
        HitCellTest(r, c)
        DragPar.Row = r
        DragPar.Col = c
        If DragPar.Col < Me.privGantt.NumberOfFixedFields Then Return
        If privGantt.ItemFieldDefinitions((c - privGantt.NumberOfFixedFields) Mod privGantt.NumberOfFieldsForItem).ReadOnly Then Return
        CurrentCell = New DataGridCell(r, c)
        DoDragDrop(DragPar, DragDropEffects.Copy) ' Or DragDropEffects.Move)
    End Sub

    Private Sub MakeTableStyles(ByVal sender As Object, ByVal e As System.EventArgs)
        AddCustomDataTableStyle()
    End Sub

    Private Sub objDataGridEx_DragOver(ByVal sender As Object, ByVal e As System.Windows.Forms.DragEventArgs) Handles MyBase.DragOver, MyBase.DragEnter
        If Not sender Is Me Then
            e.Effect = DragDropEffects.None
            Return
        End If
        Dim R, C As Integer
        HitCellTest(R, C)
        Dim Data As DragDropPars
        Data = CType(e.Data.GetData(GetType(DragDropPars)), DragDropPars)
        If privGantt.GetItemType(C) <> privGantt.GetItemType(Data.Col) Then
            e.Effect = DragDropEffects.None
            Return
        End If
        Dim shift As Integer = e.KeyState And 4
        If Not e.Data.GetData(GetType(DragDropPars)) Is Nothing Then
            If shift <> 0 Then
                e.Effect = DragDropEffects.None 'e.Effect = DragDropEffects.Move
            Else
                e.Effect = DragDropEffects.Copy
            End If
        Else
            e.Effect = DragDropEffects.None
        End If
    End Sub

    Private Sub objDataGridEx_DragDrop(ByVal sender As Object, ByVal e As System.Windows.Forms.DragEventArgs) Handles MyBase.DragDrop
        Dim R, C As Integer
        HitCellTest(R, C)
        If R >= 0 And C >= 0 Then
            Dim Data As DragDropPars
            Data = CType(e.Data.GetData(GetType(DragDropPars)), DragDropPars)
            If privGantt.GetItemType(C) = privGantt.GetItemType(Data.Col) And _
                (C <> Data.Col Or R <> Data.Row) Then
                Dim CellEf As CellDragDropEffect
                If (e.Effect And DragDropEffects.Move) = DragDropEffects.Move Then
                    CellEf = CellDragDropEffect.Move
                Else
                    CellEf = CellDragDropEffect.Copy
                End If
                Dim Cea As New CellDragDropEventArgs(CellEf, privGantt.GetItemType(C))
                Cea.Source.Row = Data.Row
                Cea.Source.Column = Data.Col
                Cea.Destination.Row = R
                Cea.Destination.Column = C
                RaiseEvent CellDragDrop(Me, Cea)
                CurrentCell = New DataGridCell(DragPar.Row, DragPar.Col)
            Else
                Dim Cea As New CellDragDropEventArgs(CellDragDropEffect.None, Nothing)
                Cea.Source.Row = Data.Row
                Cea.Source.Column = Data.Col
                Cea.Destination.Row = R
                Cea.Destination.Column = C
                RaiseEvent InvalidCellDragDrop(Me, Cea)
            End If
        End If
    End Sub

    Public Delegate Sub CDragDrop(ByVal Source As Object, ByVal Args As CellDragDropEventArgs)

    Public Event CellDragDrop As CDragDrop
    Public Event InvalidCellDragDrop As CDragDrop

End Class
